local bossfight = false
local bossfightended = false
local rank

local rankvalues = { 25,  20,  15,  8,   2,   0,  -1 }
local ranknames  = { "D", "C", "B", "A", "S", "P" }

local cv_pizzabossdebug = CV_RegisterVar({
    name = "pizzabossdebug",
    defaultvalue = "Off",
    possiblevalue = CV_OnOff,
})

local function dprint(...)
    if cv_pizzabossdebug.value then
        print(...)
    end
end

local function getRank(hurt_amount)
    local i = 2

    while hurt_amount <= rankvalues[i] do
        i = i + 1
    end

    return i - 1
end

local function resetHurtCount()
    for player in players.iterate do
        player.been_hurt = 0
    end
end

rawset(_G, "PT_StartBossFight", function()
    dprint("Starting boss fight")

    bossfight = true
    bossfightended = false

    resetHurtCount()
end)

rawset(_G, "PT_EndBossFight", function()
    dprint("Ending boss fight")

    bossfight = false
    bossfightended = true

    local hurt_amount = 0

    for player in players.iterate do
        if not player.spectator then
            P_DoPlayerExit(player)
        end

        if player.been_hurt == nil then
            player.been_hurt = 0
        end

        dprint((player.name or "???").." has been hurt "..player.been_hurt.." times")
        hurt_amount = hurt_amount + player.been_hurt
    end

    rank = getRank(hurt_amount)

    dprint("Party got the "..ranknames[rank].." rank")
end)

rawset(_G, "PT_BossFightRunning", function()
    return bossfight
end)

function A_StartBossFight(actor)
    PT_StartBossFight()
end

function A_EndBossFight(actor)
    PT_EndBossFight()
end

local function onPlayerHurt(player)
    if player.been_hurt == nil then player.been_hurt = 0 end

    dprint((player.name or "???").." has been hurt")
    player.been_hurt = player.been_hurt + 1
end
addHook("PlayerSpin", onPlayerHurt)
addHook("PlayerSquish", onPlayerHurt)
addHook("PlayerExplode", onPlayerHurt)

addHook("MapLoad", function()
    dprint("MapLoad called, reset all boss fight variables")

    bossfight = false
    bossfightended = false
    rank = nil
end)

addHook("NetVars", function(net)
    bossfight = net(bossfight)
    bossfightended = net(bossfightended)
    rank = net(rank)
end)

hud.add(function(v, player)
    if not bossfightended then return end

    local w = v.stringWidth(ranknames[rank])

    local x = 160 - w/2
    local y = 90

    v.drawString(x, y, ranknames[rank])
end)
